Do not clear enable during a context reset if a schedule enable is in
flight. This can occur if the context reset during a request
cancellation, clears the offending request, and then enables scheduling.

Fixes: 62eaf0ae217d ("drm/i915/guc: Support request cancellation")
Signed-off-by: Matthew Brost <matthew.br...@intel.com>
Cc: <sta...@vger.kernel.org
---
 .../gpu/drm/i915/gt/uc/intel_guc_submission.c    | 16 +++++++++++-----
 1 file changed, 11 insertions(+), 5 deletions(-)

diff --git a/drivers/gpu/drm/i915/gt/uc/intel_guc_submission.c 
b/drivers/gpu/drm/i915/gt/uc/intel_guc_submission.c
index e5eb2df11b4a..4631b15eccaf 100644
--- a/drivers/gpu/drm/i915/gt/uc/intel_guc_submission.c
+++ b/drivers/gpu/drm/i915/gt/uc/intel_guc_submission.c
@@ -831,17 +831,23 @@ __unwind_incomplete_requests(struct intel_context *ce)
 static void __guc_reset_context(struct intel_context *ce, bool stalled)
 {
        struct i915_request *rq;
+       unsigned long flags;
        u32 head;
 
        intel_context_get(ce);
 
        /*
         * GuC will implicitly mark the context as non-schedulable
         * when it sends the reset notification. Make sure our state
         * reflects this change. The context will be marked enabled
         * on resubmission.
while
+        * this reset was inflight. If a scheduling enable is already is in
+        * flight do not clear the enable.
         */
-       clr_context_enabled(ce);
+       spin_lock_irqsave(&ce->guc_state.lock, flags);
+       if (!context_pending_enable(ce))
+               clr_context_enabled(ce);
+       spin_unlock_irqrestore(&ce->guc_state.lock, flags);
 
        rq = intel_context_find_active_request(ce);
        if (!rq) {
@@ -831,17 +831,23 @@ __unwind_incomplete_requests(struct intel_context *ce)
 
        /*
-        * GuC will implicitly mark the context as non-schedulable
-        * when it sends the reset notification. Make sure our state
-        * reflects this change. The context will be marked enabled
-        * on resubmission.
+        * GuC will implicitly mark the context as non-schedulable when it sends
+        * the reset notification. Make sure our state reflects this change. The
+        * context will be marked enabled on resubmission. A small window exists
+        * where the context could be block & unblocked (scheduling enable) 
while
-- 
2.32.0

Reply via email to